@@ -31,6 +31,7 @@ from ryu.lib import mac
 from ryu.lib import dpid as dpid_lib
 from ryu.lib import ofctl_v1_0
 from ryu.lib import ofctl_v1_2
+from ryu.lib.packet import packet
 from ryu.ofproto import ether
 from ryu.ofproto import inet
 from ryu.ofproto import ofproto_v1_0
@@ -38,10 +39,14 @@ from ryu.ofproto import ofproto_v1_2
 from ryu.ofproto import ofproto_v1_2_parser
 
 
-LOG = logging.getLogger('ryu.app.firewall')
-
-
-# REST API
+#=============================
+#          REST API
+#=============================
+#
+#  Note: specify switch and vlan group, as follows.
+#   {switch-id} : 'all' or switchID
+#   {vlan-id}   : 'all' or vlanID
+#
 #
 ## about Firewall status
 #
@@ -50,39 +55,96 @@ LOG = logging.getLogger('ryu.app.firewall')
 #
 # set enable the firewall switches
 # PUT /firewall/module/enable/{switch-id}
-#  {switch-id} is 'all' or switchID
 #
 # set disable the firewall switches
 # PUT /firewall/module/disable/{switch-id}
-#  {switch-id} is 'all' or switchID
+#
+#
+## about Firewall logs
+#
+# get log status of all firewall switches
+# GET /firewall/log/status
+#
+# set log enable the firewall switches
+# PUT /firewall/log/enable/{switch-id}
+#
+# set log disable the firewall switches
+# PUT /firewall/log/disable/{switch-id}
 #
 #
 ## about Firewall rules
 #
 # get rules of the firewall switches
+# * for no vlan
 # GET /firewall/rules/{switch-id}
-#  {switch-id} is 'all' or switchID
+#
+# * for specific vlan group
+# GET /firewall/rules/{switch-id}/{vlan-id}
+#
 #
 # set a rule to the firewall switches
+# * for no vlan
 # POST /firewall/rules/{switch-id}
-#  {switch-id} is 'all' or switchID
+#
+# * for specific vlan group
+# POST /firewall/rules/{switch-id}/{vlan-id}
+#
+#  request body format:
+#   {"<field1>":"<value1>", "<field2>":"<value2>",...}
+#
+#     <field>  : <value>
+#    "priority": "0 to 65533"
+#    "in_port" : "<int>"
+#    "dl_src"  : "<xx:xx:xx:xx:xx:xx>"
+#    "dl_dst"  : "<xx:xx:xx:xx:xx:xx>"
+#    "dl_type" : "<ARP or IPv4>"
+#    "nw_src"  : "<A.B.C.D/M>"
+#    "nw_dst"  : "<A.B.C.D/M>"
+#    "nw_proto": "<TCP or UDP or ICMP>"
+#    "tp_src"  : "<int>"
+#    "tp_dst"  : "<int>"
+#    "actions" : "<ALLOW or DENY>"
+#
+#   Note: specifying nw_src/nw_dst
+#         without specifying dl-type as "ARP" or "IPv4"
+#         will automatically set dl-type as "IPv4".
+#
+#   Note: When "priority" has not been set up,
+#         "0" is set to "priority".
+#
+#   Note: When "actions" has not been set up,
+#         "ALLOW" is set to "actions".
+#
 #
 # delete a rule of the firewall switches from ruleID
+# * for no vlan
 # DELETE /firewall/rules/{switch-id}
-#  {switch-id} is 'all' or switchID
+#
+# * for specific vlan group
+# DELETE /firewall/rules/{switch-id}/{vlan-id}
+#
+#  request body format:
+#   {"<field>":"<value>"}
+#
+#     <field>  : <value>
+#    "rule_id" : "<int>" or "all"
 #
 
-OK = 0
-NG = -1
 
 SWITCHID_PATTERN = dpid_lib.DPID_PATTERN + r'|all'
+VLANID_PATTERN = r'[0-9]{1,4}|all'
 
 REST_ALL = 'all'
 REST_SWITCHID = 'switch_id'
+REST_VLANID = 'vlan_id'
 REST_RULE_ID = 'rule_id'
 REST_STATUS = 'status'
+REST_LOG_STATUS = 'log_status'
 REST_STATUS_ENABLE = 'enable'
 REST_STATUS_DISABLE = 'disable'
+REST_COMMAND_RESULT = 'command_result'
+REST_ACL = 'access_control_list'
+REST_RULES = 'rules'
 REST_COOKIE = 'cookie'
 REST_PRIORITY = 'priority'
 REST_MATCH = 'match'
@@ -92,6 +154,7 @@ REST_DST_MAC = 'dl_dst'
 REST_DL_TYPE = 'dl_type'
 REST_DL_TYPE_ARP = 'ARP'
 REST_DL_TYPE_IPV4 = 'IPv4'
+REST_DL_VLAN = 'dl_vlan'
 REST_SRC_IP = 'nw_src'
 REST_DST_IP = 'nw_dst'
 REST_NW_PROTO = 'nw_proto'
@@ -103,12 +166,20 @@ REST_TP_DST = 'tp_dst'
 REST_ACTION = 'actions'
 REST_ACTION_ALLOW = 'ALLOW'
 REST_ACTION_DENY = 'DENY'
+REST_ACTION_PACKETIN = 'PACKETIN'
 
 
 STATUS_FLOW_PRIORITY = ofproto_v1_2_parser.UINT16_MAX
 ARP_FLOW_PRIORITY = ofproto_v1_2_parser.UINT16_MAX - 1
+LOG_FLOW_PRIORITY = 0
+ACL_FLOW_PRIORITY_MIN = LOG_FLOW_PRIORITY + 1
 ACL_FLOW_PRIORITY_MAX = ofproto_v1_2_parser.UINT16_MAX - 2
 
+VLANID_NONE = 0
+VLANID_MIN = 2
+VLANID_MAX = 4094
+COOKIE_SHIFT_VLANID = 32
+
 
 class RestFirewallAPI(app_manager.RyuApp):
 
@@ -120,6 +191,10 @@ class RestFirewallAPI(app_manager.RyuApp):
 
     def __init__(self, *args, **kwargs):
         super(RestFirewallAPI, self).__init__(*args, **kwargs)
+
+        # logger configure
+        FirewallController.set_logger(self.logger)
+
         self.dpset = kwargs['dpset']
         wsgi = kwargs['wsgi']
         self.waiters = {}
@@ -130,8 +205,10 @@ class RestFirewallAPI(app_manager.RyuApp):
         mapper = wsgi.mapper
         wsgi.registory['FirewallController'] = self.data
         path = '/firewall'
-        requirements = {'switchid': SWITCHID_PATTERN}
+        requirements = {'switchid': SWITCHID_PATTERN,
+                        'vlanid': VLANID_PATTERN}
 
+        # for firewall status
         uri = path + '/module/status'
         mapper.connect('firewall', uri,
                        controller=FirewallController, action='get_status',
@@ -149,6 +226,25 @@ class RestFirewallAPI(app_manager.RyuApp):
                        conditions=dict(method=['PUT']),
                        requirements=requirements)
 
+        # for firewall logs
+        uri = path + '/log/status'
+        mapper.connect('firewall', uri,
+                       controller=FirewallController, action='get_log_status',
+                       conditions=dict(method=['GET']))
+
+        uri = path + '/log/enable/{switchid}'
+        mapper.connect('firewall', uri,
+                       controller=FirewallController, action='set_log_enable',
+                       conditions=dict(method=['PUT']),
+                       requirements=requirements)
+
+        uri = path + '/log/disable/{switchid}'
+        mapper.connect('firewall', uri,
+                       controller=FirewallController, action='set_log_disable',
+                       conditions=dict(method=['PUT']),
+                       requirements=requirements)
+
+        # for no VLAN data
         uri = path + '/rules/{switchid}'
         mapper.connect('firewall', uri,
                        controller=FirewallController, action='get_rules',
@@ -165,6 +261,23 @@ class RestFirewallAPI(app_manager.RyuApp):
                        conditions=dict(method=['DELETE']),
                        requirements=requirements)
 
+        # for VLAN data
+        uri += '/{vlanid}'
+        mapper.connect('firewall', uri, controller=FirewallController,
+                       action='get_vlan_rules',
+                       conditions=dict(method=['GET']),
+                       requirements=requirements)
+
+        mapper.connect('firewall', uri, controller=FirewallController,
+                       action='set_vlan_rule',
+                       conditions=dict(method=['POST']),
+                       requirements=requirements)
+
+        mapper.connect('firewall', uri, controller=FirewallController,
+                       action='delete_vlan_rule',
+                       conditions=dict(method=['DELETE']),
+                       requirements=requirements)
+
     def stats_reply_handler(self, ev):
         msg = ev.msg
         dp = msg.datapath
@@ -198,18 +311,9 @@ class RestFirewallAPI(app_manager.RyuApp):
     def stats_reply_handler_v1_2(self, ev):
         self.stats_reply_handler(ev)
 
-
-class FirewallOfs(object):
-    def __init__(self, dp):
-        super(FirewallOfs, self).__init__()
-        self.dp = dp
-        self.ctl = FirewallOfctl(dp)
-        self.cookie = 0
-
-    def get_cookie(self):
-        self.cookie += 1
-        self.cookie &= ofproto_v1_2_parser.UINT64_MAX
-        return self.cookie
+    @set_ev_cls(ofp_event.EventOFPPacketIn, MAIN_DISPATCHER)
+    def packet_in_handler(self, ev):
+        FirewallController.packet_in_handler(ev.msg)
 
 
 class FirewallOfsList(dict):
@@ -241,151 +345,203 @@ class FirewallOfsList(dict):
 class FirewallController(ControllerBase):
 
     _OFS_LIST = FirewallOfsList()
+    _LOGGER = None
 
     def __init__(self, req, link, data, **config):
         super(FirewallController, self).__init__(req, link, data, **config)
         self.dpset = data['dpset']
         self.waiters = data['waiters']
 
+    @classmethod
+    def set_logger(cls, logger):
+        cls._LOGGER = logger
+        cls._LOGGER.propagate = False
+        hdlr = logging.StreamHandler()
+        fmt_str = '[FW][%(levelname)s] %(message)s'
+        hdlr.setFormatter(logging.Formatter(fmt_str))
+        cls._LOGGER.addHandler(hdlr)
+
     @staticmethod
     def regist_ofs(dp):
+        dpid_str = dpid_lib.dpid_to_str(dp.id)
         try:
-            f_ofs = FirewallOfs(dp)
+            f_ofs = Firewall(dp)
         except OFPUnknownVersion, message:
-            mes = 'dpid=%s : %s' % (dpid_lib.dpid_to_str(dp.id), message)
-            LOG.info(mes)
+            FirewallController._LOGGER.info('dpid=%s: %s',
+                                            dpid_str, message)
             return
 
         FirewallController._OFS_LIST.setdefault(dp.id, f_ofs)
 
-        f_ofs.ctl.set_disable_flow()
-        f_ofs.ctl.set_arp_flow()
-        LOG.info('dpid=%s : Join as firewall switch.' %
-                 dpid_lib.dpid_to_str(dp.id))
+        f_ofs.set_disable_flow()
+        f_ofs.set_arp_flow()
+        f_ofs.set_log_enable()
+        FirewallController._LOGGER.info('dpid=%s: Join as firewall.',
+                                        dpid_str)
 
     @staticmethod
     def unregist_ofs(dp):
         if dp.id in FirewallController._OFS_LIST:
             del FirewallController._OFS_LIST[dp.id]
-            LOG.info('dpid=%s : Leave firewall switch.' %
-                     dpid_lib.dpid_to_str(dp.id))
+            FirewallController._LOGGER.info('dpid=%s: Leave firewall.',
+                                            dpid_lib.dpid_to_str(dp.id))
 
     # GET /firewall/module/status
     def get_status(self, req, **_kwargs):
-        try:
-            dps = self._OFS_LIST.get_ofs(REST_ALL)
-        except ValueError, message:
-            return Response(status=400, body=str(message))
-
-        msgs = {}
-        for f_ofs in dps.values():
-            status = f_ofs.ctl.get_status(self.waiters)
-            msgs.update(status)
-
-        body = json.dumps(msgs)
-        return Response(content_type='application/json', body=body)
+        return self._access_module(REST_ALL, 'get_status',
+                                   waiters=self.waiters)
 
     # POST /firewall/module/enable/{switchid}
     def set_enable(self, req, switchid, **_kwargs):
-        try:
-            dps = self._OFS_LIST.get_ofs(switchid)
-        except ValueError, message:
-            return Response(status=400, body=str(message))
-
-        msgs = {}
-        for f_ofs in dps.values():
-            msg = f_ofs.ctl.set_enable_flow()
-            msgs.update(msg)
-
-        body = json.dumps(msgs)
-        return Response(content_type='application/json', body=body)
+        return self._access_module(switchid, 'set_enable_flow')
 
     # POST /firewall/module/disable/{switchid}
     def set_disable(self, req, switchid, **_kwargs):
+        return self._access_module(switchid, 'set_disable_flow')
+
+    # GET /firewall/log/status
+    def get_log_status(self, dummy, **_kwargs):
+        return self._access_module(REST_ALL, 'get_log_status',
+                                   waiters=self.waiters)
+
+    # PUT /firewall/log/enable/{switchid}
+    def set_log_enable(self, dummy, switchid, **_kwargs):
+        return self._access_module(switchid, 'set_log_enable')
+
+    # PUT /firewall/log/disable/{switchid}
+    def set_log_disable(self, dummy, switchid, **_kwargs):
+        return self._access_module(switchid, 'set_log_disable')
+
+    def _access_module(self, switchid, func, waiters=None):
         try:
             dps = self._OFS_LIST.get_ofs(switchid)
         except ValueError, message:
             return Response(status=400, body=str(message))
 
-        msgs = {}
+        msgs = []
         for f_ofs in dps.values():
-            msg = f_ofs.ctl.set_disable_flow()
-            msgs.update(msg)
+            function = getattr(f_ofs, func)
+            msg = function() if waiters is None else function(waiters)
+            msgs.append(msg)
 
         body = json.dumps(msgs)
         return Response(content_type='application/json', body=body)
 
     # GET /firewall/rules/{switchid}
     def get_rules(self, req, switchid, **_kwargs):
-        try:
-            dps = self._OFS_LIST.get_ofs(switchid)
-        except ValueError, message:
-            return Response(status=400, body=str(message))
+        return self._get_rules(switchid)
 
-        msgs = {}
-        for f_ofs in dps.values():
-            rules = f_ofs.ctl.get_rules(self.waiters)
-            msgs.update(rules)
-
-        body = json.dumps(msgs)
-        return Response(content_type='application/json', body=body)
+    # GET /firewall/rules/{switchid}/{vlanid}
+    def get_vlan_rules(self, req, switchid, vlanid, **_kwargs):
+        return self._get_rules(switchid, vlan_id=vlanid)
 
     # POST /firewall/rules/{switchid}
     def set_rule(self, req, switchid, **_kwargs):
-        try:
-            rule = eval(req.body)
-        except SyntaxError:
-            LOG.debug('invalid syntax %s', req.body)
-            return Response(status=400)
+        return self._set_rule(req, switchid)
 
-        try:
-            dps = self._OFS_LIST.get_ofs(switchid)
-        except ValueError, message:
-            return Response(status=400, body=str(message))
-
-        msgs = {}
-        for f_ofs in dps.values():
-            try:
-                msg = f_ofs.ctl.set_rule(f_ofs.get_cookie(), rule)
-                msgs.update(msg)
-            except ValueError, message:
-                return Response(status=400, body=str(message))
-
-        body = json.dumps(msgs)
-        return Response(content_type='application/json', body=body)
+    # POST /firewall/rules/{switchid}/{vlanid}
+    def set_vlan_rule(self, req, switchid, vlanid, **_kwargs):
+        return self._set_rule(req, switchid, vlan_id=vlanid)
 
     # DELETE /firewall/rules/{switchid}
     def delete_rule(self, req, switchid, **_kwargs):
+        return self._delete_rule(req, switchid)
+
+    # DELETE /firewall/rules/{switchid}/{vlanid}
+    def delete_vlan_rule(self, req, switchid, vlanid, **_kwargs):
+        return self._delete_rule(req, switchid, vlan_id=vlanid)
+
+    def _get_rules(self, switchid, vlan_id=VLANID_NONE):
         try:
-            ruleid = eval(req.body)
+            dps = self._OFS_LIST.get_ofs(switchid)
+            vid = FirewallController._conv_toint_vlanid(vlan_id)
+        except ValueError, message:
+            return Response(status=400, body=str(message))
+
+        msgs = []
+        for f_ofs in dps.values():
+            rules = f_ofs.get_rules(self.waiters, vid)
+            msgs.append(rules)
+
+        body = json.dumps(msgs)
+        return Response(content_type='application/json', body=body)
+
+    def _set_rule(self, req, switchid, vlan_id=VLANID_NONE):
+        try:
+            rule = eval(req.body)
         except SyntaxError:
-            LOG.debug('invalid syntax %s', req.body)
+            FirewallController._LOGGER.debug('invalid syntax %s', req.body)
             return Response(status=400)
 
         try:
             dps = self._OFS_LIST.get_ofs(switchid)
+            vid = FirewallController._conv_toint_vlanid(vlan_id)
         except ValueError, message:
             return Response(status=400, body=str(message))
 
-        msgs = {}
+        msgs = []
         for f_ofs in dps.values():
             try:
-                msg = f_ofs.ctl.delete_rule(ruleid, self.waiters)
-                msgs.update(msg)
+                msg = f_ofs.set_rule(rule, vid)
+                msgs.append(msg)
             except ValueError, message:
                 return Response(status=400, body=str(message))
 
         body = json.dumps(msgs)
         return Response(content_type='application/json', body=body)
 
+    def _delete_rule(self, req, switchid, vlan_id=VLANID_NONE):
+        try:
+            ruleid = eval(req.body)
+        except SyntaxError:
+            FirewallController._LOGGER.debug('invalid syntax %s', req.body)
+            return Response(status=400)
 
-class FirewallOfctl(object):
+        try:
+            dps = self._OFS_LIST.get_ofs(switchid)
+            vid = FirewallController._conv_toint_vlanid(vlan_id)
+        except ValueError, message:
+            return Response(status=400, body=str(message))
+
+        msgs = []
+        for f_ofs in dps.values():
+            try:
+                msg = f_ofs.delete_rule(ruleid, self.waiters, vid)
+                msgs.append(msg)
+            except ValueError, message:
+                return Response(status=400, body=str(message))
+
+        body = json.dumps(msgs)
+        return Response(content_type='application/json', body=body)
+
+    @staticmethod
+    def _conv_toint_vlanid(vlan_id):
+        if vlan_id != REST_ALL:
+            vlan_id = int(vlan_id)
+            if (vlan_id != VLANID_NONE and
+                    (vlan_id < VLANID_MIN or VLANID_MAX < vlan_id)):
+                msg = 'Invalid {vlan_id} value. Set [%d-%d]' % (VLANID_MIN,
+                                                                VLANID_MAX)
+                raise ValueError(msg)
+        return vlan_id
+
+    @staticmethod
+    def packet_in_handler(msg):
+        pkt = packet.Packet(msg.data)
+        dpid_str = dpid_lib.dpid_to_str(msg.datapath.id)
+        FirewallController._LOGGER.info('dpid=%s: Blocked packet = %s',
+                                        dpid_str, pkt)
+
+
+class Firewall(object):
 
     _OFCTL = {ofproto_v1_0.OFP_VERSION: ofctl_v1_0,
               ofproto_v1_2.OFP_VERSION: ofctl_v1_2}
 
     def __init__(self, dp):
-        super(FirewallOfctl, self).__init__()
+        super(Firewall, self).__init__()
+        self.vlan_list = {}
+        self.vlan_list[VLANID_NONE] = 0  # for VLAN=None
         self.dp = dp
         version = dp.ofproto.OFP_VERSION
 
@@ -394,6 +550,42 @@ class FirewallOfctl(object):
 
         self.ofctl = self._OFCTL[version]
 
+    def _update_vlan_list(self, vlan_list):
+        for vlan_id in self.vlan_list.keys():
+            if vlan_id is not VLANID_NONE and vlan_id not in vlan_list:
+                del self.vlan_list[vlan_id]
+
+    def _get_cookie(self, vlan_id):
+        if vlan_id == REST_ALL:
+            vlan_ids = self.vlan_list.keys()
+        else:
+            vlan_ids = [vlan_id]
+
+        cookie_list = []
+        for vlan_id in vlan_ids:
+            self.vlan_list.setdefault(vlan_id, 0)
+            self.vlan_list[vlan_id] += 1
+            self.vlan_list[vlan_id] &= ofproto_v1_2_parser.UINT32_MAX
+            cookie = (vlan_id << COOKIE_SHIFT_VLANID) + \
+                self.vlan_list[vlan_id]
+            cookie_list.append([cookie, vlan_id])
+
+        return cookie_list
+
+    @staticmethod
+    def _cookie_to_ruleid(cookie):
+        return cookie & ofproto_v1_2_parser.UINT32_MAX
+
+    # REST command template
+    def rest_command(func):
+        def _rest_command(*args, **kwargs):
+            key, value = func(*args, **kwargs)
+            switch_id = dpid_lib.dpid_to_str(args[0].dp.id)
+            return {REST_SWITCHID: switch_id,
+                    key: value}
+        return _rest_command
+
+    @rest_command
     def get_status(self, waiters):
         msgs = self.ofctl.get_flow_stats(self.dp, waiters)
 
@@ -404,11 +596,9 @@ class FirewallOfctl(object):
                 if flow_stat['priority'] == STATUS_FLOW_PRIORITY:
                     status = REST_STATUS_DISABLE
 
-        msg = {REST_STATUS: status}
-        switch_id = '%s: %s' % (REST_SWITCHID,
-                                dpid_lib.dpid_to_str(self.dp.id))
-        return {switch_id: msg}
+        return REST_STATUS, status
 
+    @rest_command
     def set_disable_flow(self):
         cookie = 0
         priority = STATUS_FLOW_PRIORITY
@@ -422,10 +612,9 @@ class FirewallOfctl(object):
 
         msg = {'result': 'success',
                'details': 'firewall stopped.'}
-        switch_id = '%s: %s' % (REST_SWITCHID,
-                                dpid_lib.dpid_to_str(self.dp.id))
-        return {switch_id: msg}
+        return REST_COMMAND_RESULT, msg
 
+    @rest_command
     def set_enable_flow(self):
         cookie = 0
         priority = STATUS_FLOW_PRIORITY
@@ -439,9 +628,48 @@ class FirewallOfctl(object):
 
         msg = {'result': 'success',
                'details': 'firewall running.'}
-        switch_id = '%s: %s' % (REST_SWITCHID,
-                                dpid_lib.dpid_to_str(self.dp.id))
-        return {switch_id: msg}
+        return REST_COMMAND_RESULT, msg
+
+    @rest_command
+    def get_log_status(self, waiters):
+        msgs = self.ofctl.get_flow_stats(self.dp, waiters)
+
+        status = REST_STATUS_DISABLE
+        if str(self.dp.id) in msgs:
+            flow_stats = msgs[str(self.dp.id)]
+            for flow_stat in flow_stats:
+                if flow_stat['priority'] == LOG_FLOW_PRIORITY:
+                    if flow_stat['actions']:
+                        status = REST_STATUS_ENABLE
+
+        return REST_LOG_STATUS, status
+
+    @rest_command
+    def set_log_disable(self):
+        return self._set_log_status(False)
+
+    @rest_command
+    def set_log_enable(self):
+        return self._set_log_status(True)
+
+    def _set_log_status(self, is_enable):
+        if is_enable:
+            actions = Action.to_openflow(self.dp,
+                                         {REST_ACTION: REST_ACTION_PACKETIN})
+            details = 'Log collection started.'
+        else:
+            actions = []
+            details = 'Log collection stopped.'
+
+        flow = self._to_of_flow(cookie=0, priority=LOG_FLOW_PRIORITY,
+                                match={}, actions=actions)
+
+        cmd = self.dp.ofproto.OFPFC_ADD
+        self.ofctl.mod_flow_entry(self.dp, flow, cmd)
+
+        msg = {'result': 'success',
+               'details': details}
+        return REST_COMMAND_RESULT, msg
 
     def set_arp_flow(self):
         cookie = 0
@@ -455,12 +683,25 @@ class FirewallOfctl(object):
         cmd = self.dp.ofproto.OFPFC_ADD
         self.ofctl.mod_flow_entry(self.dp, flow, cmd)
 
-    def set_rule(self, cookie, rest):
-        priority = int(rest.get(REST_PRIORITY, 0))
+    @rest_command
+    def set_rule(self, rest, vlan_id):
+        msgs = []
+        cookie_list = self._get_cookie(vlan_id)
+        for cookie, vid in cookie_list:
+            msg = self._set_rule(cookie, rest, vid)
+            msgs.append(msg)
+        return REST_COMMAND_RESULT, msgs
 
-        if priority < 0 or ACL_FLOW_PRIORITY_MAX < priority:
-            raise ValueError('Invalid priority value. Set [0-%d]'
-                             % ACL_FLOW_PRIORITY_MAX)
+    def _set_rule(self, cookie, rest, vlan_id):
+        priority = int(rest.get(REST_PRIORITY, ACL_FLOW_PRIORITY_MIN))
+
+        if (priority < ACL_FLOW_PRIORITY_MIN
+                or ACL_FLOW_PRIORITY_MAX < priority):
+            raise ValueError('Invalid priority value. Set [%d-%d]'
+                             % (ACL_FLOW_PRIORITY_MIN, ACL_FLOW_PRIORITY_MAX))
+
+        if vlan_id:
+            rest[REST_DL_VLAN] = vlan_id
 
         match = Match.to_openflow(rest)
         actions = Action.to_openflow(self.dp, rest)
@@ -473,30 +714,44 @@ class FirewallOfctl(object):
         except:
             raise ValueError('Invalid rule parameter.')
 
+        rule_id = Firewall._cookie_to_ruleid(cookie)
         msg = {'result': 'success',
-               'details': 'Rule added. : rule_id=%d' % cookie}
+               'details': 'Rule added. : rule_id=%d' % rule_id}
 
-        switch_id = '%s: %s' % (REST_SWITCHID,
-                                dpid_lib.dpid_to_str(self.dp.id))
-        return {switch_id: msg}
+        if vlan_id != VLANID_NONE:
+            msg.setdefault(REST_VLANID, vlan_id)
+        return msg
 
-    def get_rules(self, waiters):
+    @rest_command
+    def get_rules(self, waiters, vlan_id):
         rules = {}
         msgs = self.ofctl.get_flow_stats(self.dp, waiters)
 
         if str(self.dp.id) in msgs:
             flow_stats = msgs[str(self.dp.id)]
             for flow_stat in flow_stats:
-                if (flow_stat[REST_PRIORITY] != STATUS_FLOW_PRIORITY
-                        and flow_stat[REST_PRIORITY] != ARP_FLOW_PRIORITY):
-                    rule = self._to_rest_rule(flow_stat)
-                    rules.update(rule)
+                priority = flow_stat[REST_PRIORITY]
+                if (priority != STATUS_FLOW_PRIORITY
+                        and priority != ARP_FLOW_PRIORITY
+                        and priority != LOG_FLOW_PRIORITY):
+                    vid = flow_stat[REST_MATCH].get(REST_DL_VLAN, VLANID_NONE)
+                    if vlan_id == REST_ALL or vlan_id == vid:
+                        rule = self._to_rest_rule(flow_stat)
+                        rules.setdefault(vid, [])
+                        rules[vid].append(rule)
 
-        switch_id = '%s: %s' % (REST_SWITCHID,
-                                dpid_lib.dpid_to_str(self.dp.id))
-        return {switch_id: rules}
+        get_data = []
+        for vid, rule in rules.items():
+            if vid == VLANID_NONE:
+                vid_data = {REST_RULES: rule}
+            else:
+                vid_data = {REST_VLANID: vid, REST_RULES: rule}
+            get_data.append(vid_data)
 
-    def delete_rule(self, rest, waiters):
+        return REST_ACL, get_data
+
+    @rest_command
+    def delete_rule(self, rest, waiters, vlan_id):
         try:
             if rest[REST_RULE_ID] == REST_ALL:
                 rule_id = REST_ALL
@@ -505,6 +760,7 @@ class FirewallOfctl(object):
         except:
             raise ValueError('Invalid ruleID.')
 
+        vlan_list = []
         delete_list = []
 
         msgs = self.ofctl.get_flow_stats(self.dp, waiters)
@@ -512,15 +768,22 @@ class FirewallOfctl(object):
             flow_stats = msgs[str(self.dp.id)]
             for flow_stat in flow_stats:
                 cookie = flow_stat[REST_COOKIE]
+                ruleid = Firewall._cookie_to_ruleid(cookie)
                 priority = flow_stat[REST_PRIORITY]
+                dl_vlan = flow_stat[REST_MATCH].get(REST_DL_VLAN, VLANID_NONE)
 
                 if (priority != STATUS_FLOW_PRIORITY
-                        and priority != ARP_FLOW_PRIORITY):
-                    if rule_id == REST_ALL or rule_id == cookie:
+                        and priority != ARP_FLOW_PRIORITY
+                        and priority != LOG_FLOW_PRIORITY):
+                    if ((rule_id == REST_ALL or rule_id == ruleid) and
+                            (vlan_id == dl_vlan or vlan_id == REST_ALL)):
                         match = Match.to_del_openflow(flow_stat[REST_MATCH])
                         delete_list.append([cookie, priority, match])
-                    if rule_id == cookie:
-                        break
+                    else:
+                        if dl_vlan not in vlan_list:
+                            vlan_list.append(dl_vlan)
+
+        self._update_vlan_list(vlan_list)
 
         if len(delete_list) == 0:
             msg_details = 'Rule is not exist.'
@@ -531,18 +794,27 @@ class FirewallOfctl(object):
         else:
             cmd = self.dp.ofproto.OFPFC_DELETE_STRICT
             actions = []
-            msg_details = 'Rule deleted. : ruleID='
+            delete_ids = {}
             for cookie, priority, match in delete_list:
                 flow = self._to_of_flow(cookie=cookie, priority=priority,
                                         match=match, actions=actions)
                 self.ofctl.mod_flow_entry(self.dp, flow, cmd)
-                msg_details += '%d,' % cookie
-            msg = {'result': 'success',
-                   'details': msg_details}
 
-        switch_id = '%s: %s' % (REST_SWITCHID,
-                                dpid_lib.dpid_to_str(self.dp.id))
-        return {switch_id: msg}
+                vid = match.get(REST_DL_VLAN, VLANID_NONE)
+                rule_id = Firewall._cookie_to_ruleid(cookie)
+                delete_ids.setdefault(vid, '')
+                delete_ids[vid] += (('%d' if delete_ids[vid] == ''
+                                     else ',%d') % rule_id)
+
+            msg = []
+            for vid, rule_ids in delete_ids.items():
+                del_msg = {'result': 'success',
+                           'details': 'Rule deleted. : ruleID=%s' % rule_ids}
+                if vid != VLANID_NONE:
+                    del_msg.setdefault(REST_VLANID, vid)
+                msg.append(del_msg)
+
+        return REST_COMMAND_RESULT, msg
 
     def _to_of_flow(self, cookie, priority, match, actions):
         flow = {'cookie': cookie,
@@ -555,12 +827,12 @@ class FirewallOfctl(object):
         return flow
 
     def _to_rest_rule(self, flow):
-        rule_id = '%s: %d' % (REST_RULE_ID, flow[REST_COOKIE])
-
-        rule = {REST_PRIORITY: flow[REST_PRIORITY]}
+        ruleid = Firewall._cookie_to_ruleid(flow[REST_COOKIE])
+        rule = {REST_RULE_ID: ruleid}
+        rule.update({REST_PRIORITY: flow[REST_PRIORITY]})
         rule.update(Match.to_rest(flow))
         rule.update(Action.to_rest(flow))
-        return {rule_id: rule}
+        return rule
 
 
 class Match(object):
@@ -666,6 +938,10 @@ class Action(object):
                        'port': out_port}]
         elif value == REST_ACTION_DENY:
             action = []
+        elif value == REST_ACTION_PACKETIN:
+            out_port = dp.ofproto.OFPP_CONTROLLER
+            action = [{'type': 'OUTPUT',
+                       'port': out_port}]
         else:
             raise ValueError('Invalid action type.')
 
