@@ -60,7 +60,8 @@ def to_actions(dp, acts):
             ethertype = int(a.get('ethertype'))
             actions.append(parser.OFPActionPushMpls(ethertype))
         elif action_type == 'POP_MPLS':
-            actions.append(parser.OFPActionPopMpls())
+            ethertype = int(a.get('ethertype'))
+            actions.append(parser.OFPActionPopMpls(ethertype))
         elif action_type == 'SET_QUEUE':
             queue_id = int(a.get('queue_id'))
             actions.append(parser.OFPActionSetQueue(queue_id))
@@ -254,7 +255,7 @@ def mod_flow_entry(dp, flow, cmd):
     out_group = int(flow.get('out_group', dp.ofproto.OFPG_ANY))
     flags = int(flow.get('flags', 0))
     match = to_match(dp, flow.get('match', {}))
-    inst = to_actions(dp, flow.get('actions', {}))
+    inst = to_actions(dp, flow.get('actions', []))
 
     flow_mod = dp.ofproto_parser.OFPFlowMod(
         dp, cookie, cookie_mask, table_id, cmd, idle_timeout,
