@@ -97,6 +97,15 @@ class Packet(object):
         assert issubclass(protocol, packet_base.PacketBase)
         return [p for p in self.protocols if isinstance(p, protocol)]
 
+    def get_protocol(self, protocol):
+        """Returns the firstly found protocol that matches to the
+        specified protocol.
+        """
+        result = self.get_protocols(protocol)
+        if len(result) > 0:
+            return result[0]
+        return None
+
     def next(self):
         try:
             p = self.protocols[self.protocol_idx]
@@ -107,6 +116,10 @@ class Packet(object):
         self.protocol_idx += 1
         return p
 
+    def __div__(self, trailer):
+        self.add_protocol(trailer)
+        return self
+
     def __iter__(self):
         return self
 
@@ -127,3 +140,17 @@ class Packet(object):
                 issubclass(protocol, packet_base.PacketBase)):
             return protocol in [p.__class__ for p in self.protocols]
         return protocol in self.protocols
+
+    def __str__(self):
+        return ', '.join(repr(protocol) for protocol in self.protocols)
+    __repr__ = __str__  # note: str(list) uses __repr__ for elements
+
+
+# XXX: Hack for preventing recursive import
+def _PacketBase__div__(self, trailer):
+    pkt = Packet()
+    pkt.add_protocol(self)
+    pkt.add_protocol(trailer)
+    return pkt
+
+packet_base.PacketBase.__div__ = _PacketBase__div__
