@@ -18,7 +18,7 @@
 import unittest
 import logging
 import struct
-import netaddr
+import inspect
 
 from nose.tools import ok_, eq_, nottest, raises
 from nose.plugins.skip import Skip, SkipTest
@@ -28,13 +28,16 @@ from ryu.lib.packet.packet import Packet
 from ryu.lib.packet import icmpv6
 from ryu.lib.packet.ipv6 import ipv6
 from ryu.lib.packet import packet_utils
+from ryu.lib import addrconv
 
 
 LOG = logging.getLogger(__name__)
 
 
 def icmpv6_csum(prev, buf):
-    ph = struct.pack('!16s16sI3xB', prev.src, prev.dst,
+    ph = struct.pack('!16s16sI3xB',
+                     addrconv.ipv6.text_to_bin(prev.src),
+                     addrconv.ipv6.text_to_bin(prev.dst),
                      prev.payload_length, prev.nxt)
     h = bytearray(buf)
     struct.pack_into('!H', h, 2, 0)
@@ -70,8 +73,8 @@ class Test_icmpv6_header(unittest.TestCase):
         eq_(n, None)
 
     def test_serialize(self):
-        src_ipv6 = netaddr.IPAddress('fe80::200:ff:fe00:ef').packed
-        dst_ipv6 = netaddr.IPAddress('fe80::200:ff:fe00:1').packed
+        src_ipv6 = 'fe80::200:ff:fe00:ef'
+        dst_ipv6 = 'fe80::200:ff:fe00:1'
         prev = ipv6(6, 0, 0, 4, 58, 255, src_ipv6, dst_ipv6)
 
         buf = self.icmp.serialize(bytearray(), prev)
@@ -128,8 +131,8 @@ class Test_icmpv6_echo_request(unittest.TestCase):
 
     def _test_serialize(self, echo_data=None):
         buf = self.buf + str(echo_data or '')
-        src_ipv6 = netaddr.IPAddress('3ffe:507:0:1:200:86ff:fe05:80da').packed
-        dst_ipv6 = netaddr.IPAddress('3ffe:501:0:1001::2').packed
+        src_ipv6 = '3ffe:507:0:1:200:86ff:fe05:80da'
+        dst_ipv6 = '3ffe:501:0:1001::2'
         prev = ipv6(6, 0, 0, len(buf), 64, 255, src_ipv6, dst_ipv6)
         echo_csum = icmpv6_csum(prev, buf)
 
@@ -155,6 +158,30 @@ class Test_icmpv6_echo_request(unittest.TestCase):
     def test_serialize_with_data(self):
         self._test_serialize(self.data)
 
+    def test_to_string(self):
+        ec = icmpv6.echo(self.id_, self.seq, self.data)
+        ic = icmpv6.icmpv6(self.type_, self.code, self.csum, ec)
+
+        echo_values = {'id': self.id_,
+                       'seq': self.seq,
+                       'data': self.data}
+        _echo_str = ','.join(['%s=%s' % (k, repr(echo_values[k]))
+                              for k, v in inspect.getmembers(ec)
+                              if k in echo_values])
+        echo_str = '%s(%s)' % (icmpv6.echo.__name__, _echo_str)
+
+        icmp_values = {'type_': repr(self.type_),
+                       'code': repr(self.code),
+                       'csum': repr(self.csum),
+                       'data': echo_str}
+        _ic_str = ','.join(['%s=%s' % (k, icmp_values[k])
+                            for k, v in inspect.getmembers(ic)
+                            if k in icmp_values])
+        ic_str = '%s(%s)' % (icmpv6.icmpv6.__name__, _ic_str)
+
+        eq_(str(ic), ic_str)
+        eq_(repr(ic), ic_str)
+
 
 class Test_icmpv6_echo_reply(Test_icmpv6_echo_request):
     def setUp(self):
@@ -168,16 +195,16 @@ class Test_icmpv6_neighbor_solict(unittest.TestCase):
     code = 0
     csum = 0x952d
     res = 0
-    dst = netaddr.IPAddress('3ffe:507:0:1:200:86ff:fe05:80da').packed
+    dst = '3ffe:507:0:1:200:86ff:fe05:80da'
     nd_type = 1
     nd_length = 1
-    nd_hw_src = '\x00\x60\x97\x07\x69\xea'
+    nd_hw_src = '00:60:97:07:69:ea'
     data = '\x01\x01\x00\x60\x97\x07\x69\xea'
     buf = '\x87\x00\x95\x2d\x00\x00\x00\x00' \
         + '\x3f\xfe\x05\x07\x00\x00\x00\x01' \
         + '\x02\x00\x86\xff\xfe\x05\x80\xda'
-    src_ipv6 = netaddr.IPAddress('3ffe:507:0:1:200:86ff:fe05:80da').packed
-    dst_ipv6 = netaddr.IPAddress('3ffe:501:0:1001::2').packed
+    src_ipv6 = '3ffe:507:0:1:200:86ff:fe05:80da'
+    dst_ipv6 = '3ffe:501:0:1001::2'
 
     def setUp(self):
         pass
@@ -201,7 +228,8 @@ class Test_icmpv6_neighbor_solict(unittest.TestCase):
         eq_(msg.code, self.code)
         eq_(msg.csum, self.csum)
         eq_(msg.data.res >> 29, self.res)
-        eq_(msg.data.dst, self.dst)
+        eq_(addrconv.ipv6.text_to_bin(msg.data.dst),
+            addrconv.ipv6.text_to_bin(self.dst))
         eq_(n, None)
         if data:
             nd = msg.data
@@ -232,7 +260,7 @@ class Test_icmpv6_neighbor_solict(unittest.TestCase):
         eq_(code, self.code)
         eq_(csum, nd_csum)
         eq_(res >> 29, self.res)
-        eq_(dst, self.dst)
+        eq_(dst, addrconv.ipv6.text_to_bin(self.dst))
         eq_(data, '')
 
     def test_serialize_with_data(self):
@@ -255,10 +283,45 @@ class Test_icmpv6_neighbor_solict(unittest.TestCase):
         eq_(code, self.code)
         eq_(csum, nd_csum)
         eq_(res >> 29, self.res)
-        eq_(dst, self.dst)
+        eq_(dst, addrconv.ipv6.text_to_bin(self.dst))
         eq_(nd_type, self.nd_type)
         eq_(nd_length, self.nd_length)
-        eq_(nd_hw_src, self.nd_hw_src)
+        eq_(nd_hw_src, addrconv.mac.text_to_bin(self.nd_hw_src))
+
+    def test_to_string(self):
+        nd_opt = icmpv6.nd_option_la(self.nd_hw_src)
+        nd = icmpv6.nd_neighbor(
+            self.res, self.dst, self.nd_type, self.nd_length, nd_opt)
+        ic = icmpv6.icmpv6(self.type_, self.code, self.csum, nd)
+
+        nd_opt_values = {'hw_src': self.nd_hw_src,
+                         'data': None}
+        _nd_opt_str = ','.join(['%s=%s' % (k, repr(nd_opt_values[k]))
+                                for k, v in inspect.getmembers(nd_opt)
+                                if k in nd_opt_values])
+        nd_opt_str = '%s(%s)' % (icmpv6.nd_option_la.__name__, _nd_opt_str)
+
+        nd_values = {'res': repr(nd.res),
+                     'dst': repr(self.dst),
+                     'type_': repr(self.nd_type),
+                     'length': repr(self.nd_length),
+                     'data': nd_opt_str}
+        _nd_str = ','.join(['%s=%s' % (k, nd_values[k])
+                            for k, v in inspect.getmembers(nd)
+                            if k in nd_values])
+        nd_str = '%s(%s)' % (icmpv6.nd_neighbor.__name__, _nd_str)
+
+        icmp_values = {'type_': repr(self.type_),
+                       'code': repr(self.code),
+                       'csum': repr(self.csum),
+                       'data': nd_str}
+        _ic_str = ','.join(['%s=%s' % (k, icmp_values[k])
+                            for k, v in inspect.getmembers(ic)
+                            if k in icmp_values])
+        ic_str = '%s(%s)' % (icmpv6.icmpv6.__name__, _ic_str)
+
+        eq_(str(ic), ic_str)
+        eq_(repr(ic), ic_str)
 
 
 class Test_icmpv6_neighbor_advert(Test_icmpv6_neighbor_solict):
@@ -266,11 +329,11 @@ class Test_icmpv6_neighbor_advert(Test_icmpv6_neighbor_solict):
         self.type_ = 136
         self.csum = 0xb8ba
         self.res = 7
-        self.dst = netaddr.IPAddress('3ffe:507:0:1:260:97ff:fe07:69ea').packed
+        self.dst = '3ffe:507:0:1:260:97ff:fe07:69ea'
         self.nd_type = 2
         self.nd_length = 1
         self.nd_data = None
-        self.nd_hw_src = '\x00\x60\x97\x07\x69\xea'
+        self.nd_hw_src = '00:60:97:07:69:ea'
         self.data = '\x02\x01\x00\x60\x97\x07\x69\xea'
         self.buf = '\x88\x00\xb8\xba\xe0\x00\x00\x00' \
             + '\x3f\xfe\x05\x07\x00\x00\x00\x01' \
@@ -284,10 +347,11 @@ class Test_icmpv6_router_solict(unittest.TestCase):
     res = 0
     nd_type = 1
     nd_length = 1
-    nd_data = None
-    nd_hw_src = '\x12\x2d\xa5\x6d\xbc\x0f'
+    nd_hw_src = '12:2d:a5:6d:bc:0f'
     data = '\x00\x00\x00\x00\x01\x01\x12\x2d\xa5\x6d\xbc\x0f'
     buf = '\x85\x00\x97\xd9'
+    src_ipv6 = '3ffe:507:0:1:200:86ff:fe05:80da'
+    dst_ipv6 = '3ffe:501:0:1001::2'
 
     def setUp(self):
         pass
@@ -296,7 +360,11 @@ class Test_icmpv6_router_solict(unittest.TestCase):
         pass
 
     def test_init(self):
-        pass
+        rs = icmpv6.nd_router_solicit(self.res)
+        eq_(rs.res, self.res)
+        eq_(rs.type_, None)
+        eq_(rs.length, None)
+        eq_(rs.data, None)
 
     def _test_parser(self, data=None):
         buf = self.buf + str(data or '')
@@ -305,7 +373,15 @@ class Test_icmpv6_router_solict(unittest.TestCase):
         eq_(msg.type_, self.type_)
         eq_(msg.code, self.code)
         eq_(msg.csum, self.csum)
-        eq_(msg.data, data)
+        if data is not None:
+            eq_(msg.data.res[0], self.res)
+        eq_(n, None)
+        if data:
+            rs = msg.data
+            eq_(rs.type_, self.nd_type)
+            eq_(rs.length, self.nd_length)
+            eq_(rs.data.hw_src, self.nd_hw_src)
+            eq_(rs.data.data, None)
 
     def test_parser_without_data(self):
         self._test_parser()
@@ -313,26 +389,78 @@ class Test_icmpv6_router_solict(unittest.TestCase):
     def test_parser_with_data(self):
         self._test_parser(self.data)
 
-    def _test_serialize(self, nd_data=None):
-        nd_data = str(nd_data or '')
-        buf = self.buf + nd_data
-        src_ipv6 = netaddr.IPAddress('fe80::102d:a5ff:fe6d:bc0f').packed
-        dst_ipv6 = netaddr.IPAddress('ff02::2').packed
-        prev = ipv6(6, 0, 0, len(buf), 58, 255, src_ipv6, dst_ipv6)
-        nd_csum = icmpv6_csum(prev, buf)
+    def test_serialize_without_data(self):
+        rs = icmpv6.nd_router_solicit(self.res)
+        prev = ipv6(6, 0, 0, 8, 64, 255, self.src_ipv6, self.dst_ipv6)
+        rs_csum = icmpv6_csum(prev, self.buf)
 
-        icmp = icmpv6.icmpv6(self.type_, self.code, 0, nd_data)
+        icmp = icmpv6.icmpv6(self.type_, self.code, 0, rs)
         buf = buffer(icmp.serialize(bytearray(), prev))
+
         (type_, code, csum) = struct.unpack_from(icmp._PACK_STR, buf, 0)
-        data = buf[icmp._MIN_LEN:]
+        res = struct.unpack_from(rs._PACK_STR, buf, icmp._MIN_LEN)
+        data = buf[(icmp._MIN_LEN + rs._MIN_LEN):]
 
         eq_(type_, self.type_)
         eq_(code, self.code)
-        eq_(csum, nd_csum)
-        eq_(data, nd_data)
-
-    def test_serialize_without_data(self):
-        self._test_serialize()
+        eq_(csum, rs_csum)
+        eq_(res[0], self.res)
+        eq_(data, '')
 
     def test_serialize_with_data(self):
-        self._test_serialize(self.data)
+        nd_opt = icmpv6.nd_option_la(self.nd_hw_src)
+        rs = icmpv6.nd_router_solicit(self.res, self.nd_type, self.nd_length,
+                                      nd_opt)
+        prev = ipv6(6, 0, 0, 16, 64, 255, self.src_ipv6, self.dst_ipv6)
+        rs_csum = icmpv6_csum(prev, self.buf + self.data)
+
+        icmp = icmpv6.icmpv6(self.type_, self.code, 0, rs)
+        buf = buffer(icmp.serialize(bytearray(), prev))
+
+        (type_, code, csum) = struct.unpack_from(icmp._PACK_STR, buf, 0)
+        res = struct.unpack_from(rs._PACK_STR, buf, icmp._MIN_LEN)
+        (nd_type, nd_length, nd_hw_src) = struct.unpack_from(
+            '!BB6s', buf, icmp._MIN_LEN + rs._MIN_LEN)
+        data = buf[(icmp._MIN_LEN + rs._MIN_LEN + 8):]
+
+        eq_(type_, self.type_)
+        eq_(code, self.code)
+        eq_(csum, rs_csum)
+        eq_(res[0], self.res)
+        eq_(nd_type, self.nd_type)
+        eq_(nd_length, self.nd_length)
+        eq_(nd_hw_src, addrconv.mac.text_to_bin(self.nd_hw_src))
+
+    def test_to_string(self):
+        nd_opt = icmpv6.nd_option_la(self.nd_hw_src)
+        rs = icmpv6.nd_router_solicit(
+            self.res, self.nd_type, self.nd_length, nd_opt)
+        ic = icmpv6.icmpv6(self.type_, self.code, self.csum, rs)
+
+        nd_opt_values = {'hw_src': self.nd_hw_src,
+                         'data': None}
+        _nd_opt_str = ','.join(['%s=%s' % (k, repr(nd_opt_values[k]))
+                                for k, v in inspect.getmembers(nd_opt)
+                                if k in nd_opt_values])
+        nd_opt_str = '%s(%s)' % (icmpv6.nd_option_la.__name__, _nd_opt_str)
+
+        rs_values = {'res': repr(rs.res),
+                     'type_': repr(self.nd_type),
+                     'length': repr(self.nd_length),
+                     'data': nd_opt_str}
+        _rs_str = ','.join(['%s=%s' % (k, rs_values[k])
+                            for k, v in inspect.getmembers(rs)
+                            if k in rs_values])
+        rs_str = '%s(%s)' % (icmpv6.nd_router_solicit.__name__, _rs_str)
+
+        icmp_values = {'type_': repr(self.type_),
+                       'code': repr(self.code),
+                       'csum': repr(self.csum),
+                       'data': rs_str}
+        _ic_str = ','.join(['%s=%s' % (k, icmp_values[k])
+                            for k, v in inspect.getmembers(ic)
+                            if k in icmp_values])
+        ic_str = '%s(%s)' % (icmpv6.icmpv6.__name__, _ic_str)
+
+        eq_(str(ic), ic_str)
+        eq_(repr(ic), ic_str)
