@@ -18,17 +18,15 @@
 import unittest
 import logging
 import struct
-import netaddr
 from struct import *
 from nose.tools import *
 from nose.plugins.skip import Skip, SkipTest
 from ryu.ofproto import ether, inet
-from ryu.lib import mac
-from ryu.lib.packet.ethernet import ethernet
 from ryu.lib.packet.packet import Packet
 from ryu.lib.packet.udp import udp
 from ryu.lib.packet.ipv4 import ipv4
 from ryu.lib.packet import packet_utils
+from ryu.lib import addrconv
 
 
 LOG = logging.getLogger('test_udp')
@@ -71,8 +69,8 @@ class Test_udp(unittest.TestCase):
         total_length = 0
         csum = 0
 
-        src_ip = netaddr.IPAddress('192.168.10.1').packed
-        dst_ip = netaddr.IPAddress('192.168.100.1').packed
+        src_ip = '192.168.10.1'
+        dst_ip = '192.168.100.1'
         prev = ipv4(4, 5, 0, 0, 0, 0, 0, 64,
                     inet.IPPROTO_UDP, 0, src_ip, dst_ip)
 
@@ -85,7 +83,9 @@ class Test_udp(unittest.TestCase):
         eq_(res[2], struct.calcsize(udp._PACK_STR))
 
         # checksum
-        ph = struct.pack('!4s4sBBH', src_ip, dst_ip, 0, 17, res[2])
+        ph = struct.pack('!4s4sBBH',
+                         addrconv.ipv4.text_to_bin(src_ip),
+                         addrconv.ipv4.text_to_bin(dst_ip), 0, 17, res[2])
         d = ph + buf + bytearray()
         s = packet_utils.checksum(d)
         eq_(0, s)
