@@ -25,8 +25,8 @@ from ryu.controller import ofp_event
 from ryu.controller.handler import set_ev_cls
 from ryu.controller.handler import MAIN_DISPATCHER, DEAD_DISPATCHER
 from ryu.exception import RyuException
-from ryu.lib import hub
-from ryu.lib.mac import DONTCARE, haddr_to_str
+from ryu.lib import addrconv, hub
+from ryu.lib.mac import DONTCARE_STR
 from ryu.lib.dpid import dpid_to_str, str_to_dpid
 from ryu.lib.port_no import port_no_to_str
 from ryu.lib.packet import packet, ethernet, lldp
@@ -82,7 +82,7 @@ class Port(object):
     def to_dict(self):
         return {'dpid': dpid_to_str(self.dpid),
                 'port_no': port_no_to_str(self.port_no),
-                'hw_addr': haddr_to_str(self.hw_addr),
+                'hw_addr': self.hw_addr,
                 'name': self.name.rstrip('\0')}
 
     # for Switch.del_port()
@@ -431,7 +431,7 @@ class Switches(app_manager.RyuApp):
                event.EventLinkAdd, event.EventLinkDelete]
 
     DEFAULT_TTL = 120  # unused. ignored.
-    LLDP_PACKET_LEN = len(LLDPPacket.lldp_packet(0, 0, DONTCARE, 0))
+    LLDP_PACKET_LEN = len(LLDPPacket.lldp_packet(0, 0, DONTCARE_STR, 0))
 
     LLDP_SEND_GUARD = .05
     LLDP_SEND_PERIOD_PER_PORT = .9
@@ -537,7 +537,8 @@ class Switches(app_manager.RyuApp):
                 # TODO:XXX need other versions
                 if ofproto.OFP_VERSION == ofproto_v1_0.OFP_VERSION:
                     rule = nx_match.ClsRule()
-                    rule.set_dl_dst(lldp.LLDP_MAC_NEAREST_BRIDGE)
+                    rule.set_dl_dst(addrconv.mac.text_to_bin(
+                                    lldp.LLDP_MAC_NEAREST_BRIDGE))
                     rule.set_dl_type(ETH_TYPE_LLDP)
                     actions = [ofproto_parser.OFPActionOutput(
                         ofproto.OFPP_CONTROLLER, self.LLDP_PACKET_LEN)]
